@@ -23,7 +23,11 @@ namespace common {
  * \return the transformed value.
  */
 XGBOOST_DEVICE inline float Sigmoid(float x) {
-  return 1.0f / (1.0f + expf(-x));
+  float constexpr kEps = 1e-16;  // avoid 0 div
+  x = std::min(-x, 88.7f);       // avoid exp overflow
+  auto denom = expf(x) + 1.0f + kEps;
+  auto y = 1.0f / denom;
+  return y;
 }
 
 template <typename T>
@@ -77,7 +81,7 @@ XGBOOST_DEVICE inline void Softmax(Iterator start, Iterator end) {
 
 /*!
  * \brief Find the maximum iterator within the iterators
- * \param begin The begining iterator.
+ * \param begin The beginning iterator.
  * \param end The end iterator.
  * \return the iterator point to the maximum value.
  * \tparam Iterator The type of the iterator.
@@ -107,7 +111,7 @@ inline float LogSum(float x, float y) {
 
 /*!
  * \brief perform numerically safe logsum
- * \param begin The begining iterator.
+ * \param begin The beginning iterator.
  * \param end The end iterator.
  * \return the iterator point to the maximum value.
  * \tparam Iterator The type of the iterator.
@@ -135,7 +139,7 @@ inline static bool CmpSecond(const std::pair<float, unsigned> &a,
   return a.second > b.second;
 }
 
-// Redefined here to workaround a VC bug that doesn't support overloadng for integer
+// Redefined here to workaround a VC bug that doesn't support overloading for integer
 // types.
 template <typename T>
 XGBOOST_DEVICE typename std::enable_if<
